﻿#requires -version 3.0

<#
DO NOT RUN ANY OF THESE WORKFLOWS
IN A PRODUCTION ENVIRONMENT.

THESE ARE FOR DEMONSTRATION AND EDUCATIONAL
PURPOSES ONLY

Jeffery Hicks
http://jdhitsolutions.com/blog
http://twitter.com/JeffHicks
#>

workflow Basic {

$VerbosePreference="Continue"
write-verbose -Message "Starting a basic workflow $($workflowcommandname)"

$p=Get-Process -Name svchost
$p
$p | Export-Clixml -path c:\svchosts.xml

write-verbose -Message "Ending a basic workflow"

}


Workflow RestartMe {
$log="c:\work\wflog.txt"

$t="{0} Starting {1}" -f (Get-date).TimeOfDay,$workflowcommandname

Get-ChildItem -Path $env:temp -Recurse | Measure-Object -sum -property Length | out-file -FilePath $log

#this only works for remote computers
Restart-Computer -force -Wait -for PowerShell

$t="{0} Finished {1}" -f (Get-date).TimeOfDay,$workflowcommandname
$t | Out-File -FilePath $log -Append

}


#demo InlineScript
Workflow DemoInline {

$VerbosePreference="Continue"
 Write-verbose -Message "Starting the workflow"
 Inlinescript {
    
     5..1 | foreach {$_ ; start-sleep 1}
     nbtstat -n
     
  }
  write-verbose -Message "Done"
}


#demo Foreach Parallel
Workflow DemoForEachParallel {

$VerbosePreference="Continue"
$paths="C:\work","C:\windows\SoftwareDistribution",$home

foreach -parallel ($path in $paths) {
    write-verbose -message $path
    $stat=dir -path $path -file -recurse | 
    measure-object -Property Length -sum
    $obj=New-Object -TypeName PSObject -Property @{
      Path=$Path
      TotalFiles=$stat.count
      TotalSizeMB=$stat.sum/1MB
      }
    $obj
  } #foreach
Write-verbose -message "Finished"
} #close workflow



Workflow DemoParallel {

$VerbosePreference="Continue"

write-verbose -Message "Getting data from $computername"
 
parallel {  
    #one line commands
    #scope is super critical here if you want to access variables 
    #in other scriptblocks in this workflow    
    $workflow:bios = Get-WmiObject -class win32_bios 
    $workflow:os = Get-WmiObject -class win32_operatingsystem 
    $workflow:cs = Get-WmiObject -class win32_computersystem 
    $workflow:disks = Get-WmiObject -Class win32_logicaldisk -filter "Drivetype=3" 
 } #parallel


$hash=[ordered]@{
Computername=$workflow:cs.Name
Bios=$workflow:bios
OperatingSystem=$workflow:os
ComputerSystem=$workflow:cs
Disks=$workflow:disks
}

$obj=New-Object -TypeName PSObject -Property $hash

Write-Output -InputObject $obj
write-verbose -message "Finished with WMI data"

} #close workflow 


Workflow DemoSequence {

$VerbosePreference="Continue"
write-verbose -message ("{0} starting" -f (Get-Date).TimeofDay)
 $a=10
    Sequence {
        "{0} sequence 1" -f (Get-Date).TimeOfDay
        $workflow:a++
    }
    Sequence {
        "{0} sequence 2" -f (Get-Date).TimeofDay
        $workflow:a++
        $workflow:b=100
    }

    Sequence {
        "{0} sequence 3" -f (Get-Date).TimeofDay
        $workflow:a++
        $workflow:b*=2
    }
 "a = $a"
 "b = $b"
 
write-verbose -Message ("{0} ending" -f (Get-Date).TimeOfDay)

}

Workflow DemoParallelSequence {

"{0} Starting" -f (Get-date).TimeOfDay
 $a=10

"a = $a"
"b = $b"

 Parallel {
 Write-verbose -Message "In parallel"
    Sequence {
        "{0} sequence 1" -f (get-Date).TimeOfDay
        $workflow:a++
        start-sleep -Seconds 1
    }
    Sequence {
        "{0} sequence 2" -f (get-Date).TimeOfDay
        $workflow:a++
        $workflow:b=100
        start-sleep -Seconds 1
    }

    Sequence {
        "{0} sequence 3" -f (get-Date).TimeOfDay
        $workflow:a++
        $workflow:b*=2
        start-sleep -Seconds 1
    }
 #this runs in parallel with the sequences
 "{0} Parallel" -f (Get-Date).TimeOfDay
 "a = $a"
 "b = $b"

} #parallel

#the results after Parallel
"{0} Final Results" -f (get-date).TimeOfDay
 "a = $a"
 "b = $b"
 
"{0} Ending" -f (get-date).TimeOfDay

} #close workflow


Workflow ShowCommonParam {

#demo running this from the console with tab completion

$PSComputerName
$PSCredential
$PSConnectionRetryCount
$PSActionRetryCount
$PSPersist

}

Workflow DemoUsing {

Param([string]$log="System",[int]$newest=10)

#creating a variable within the workflow
$source="Service Control Manager"

InlineScript {

    <#
    this should be the way to access out of scope variables with
    InlineScript. Although in my testing I sometimes can get the
    workflow to work without using it.
    #>
   "Getting newest {0} logs from {1} on {2}" -f $using:newest,$using:log,$pscomputername
   get-eventlog -LogName $using:log -Newest $using:newest -Source $using:source

 } #inlinescript

} #close workflow


Workflow DemoPersistence {

"{0} Starting $workflowcommandname on {1}" -f (Get-Date).TimeOfDay,$($pscomputername)

$start=Get-Date

$p=get-process -Name lsass

start-sleep -Seconds 5
$a=123
"a originally = $a"

#suspending will automatically create a job
Suspend-workflow

<#
  look at jobs
  exit PowerShell sessions
  open a new session
  Run Get-Job
  Import-module PSWorkflow
  Run Get-Job
  Resume job
  receive job results
#>

#we'll only see results from this point on when you resume the job
$a*=2

"a is finally $a"

InlineScript { $using:p}

$endtime=Get-Date

"Total elapsed time = {0}" -f ($endtime-$start)

"{0} Ending $workflowcommandname on {1}" -f (Get-Date).TimeOfDay,$($pscomputername)

}


Workflow DemoPersistence2 {

<#
1. Run with no parameters then reboot after the countdown has begun.
   Open PowerShell and check for workflows or jobs. Don't forget to
   import the PSWorkflow module. If there is a job, resume it and get
   job results. What did you get?

2. Run with workflow with -pspersist $True and repeat.

3. Run the workflow -asJob. 
   Run Get-Job
   Suspend the job with Suspend-Job
   Exit Powershell and start a new session or reboot.
   Import the PSWorkflow module
   Resume the job
   Get results. Did you use -PSPersist $True? What did you get?
#>

"{0} Starting $workflowcommandname" -f (Get-Date).TimeOfDay

$start=Get-Date

$a=0

Do {
  $a
  $a++
  Start-Sleep -milliseconds 500
} until ($a -gt 100)

$endtime=Get-Date

"Total elapsed time = {0}" -f ($endtime-$start)

"{0} Ending $workflowcommandname" -f (Get-Date).TimeOfDay

}

